
from assembly_payments.services.base import BaseService
from assembly_payments.types import BankAccount, User, BankAccountRequest


class BankAccountService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.endpoint = "/bank_accounts"

    def get(self, bank_account_id):
        response = self._execute(BankAccountService.GET, f"{self.endpoint}/{bank_account_id}")
        return BankAccount(**response['bank_accounts'])

    def get_user(self, bank_account_id):
        response = self._execute(BankAccountService.GET, f"{self.endpoint}/{bank_account_id}/users")
        return User(**response['users'])

    def create(self, **kwargs):
        data = BankAccountRequest(**kwargs)
        response = self._execute(BankAccountService.POST, self.endpoint, data=data.dict())
        return BankAccount(**response['bank_accounts'])



from assembly_payments.services.base import BaseService
from assembly_payments.types import (
    User,
    WalletAccount,
    NppDetails,
    BpayDetails,
    VirtualAccount,
    Disbursement,
    WithdrawFundsRequest,
)


class WalletAccountService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.endpoint = "/wallet_accounts"

    def get(self, wallet_account_id):
        response = self._execute(
            WalletAccountService.GET, f"{self.endpoint}/{wallet_account_id}"
        )
        return WalletAccount(**response["wallet_accounts"])

    def get_user(self, wallet_account_id):
        response = self._execute(
            WalletAccountService.GET, f"{self.endpoint}/{wallet_account_id}/users"
        )
        return User(**response["users"])

    def get_wallet_account_npp_details(self, wallet_account_id):
        response = self._execute(
            WalletAccountService.GET, f"{self.endpoint}/{wallet_account_id}/npp_details"
        )
        return NppDetails(**response["wallet_accounts"]["npp_details"])

    def get_wallet_account_bpay_details(self, wallet_account_id):
        response = self._execute(
            WalletAccountService.GET,
            f"{self.endpoint}/{wallet_account_id}/bpay_details",
        )
        return BpayDetails(**response["wallet_accounts"]["bpay_details"])

    def get_virtual_accounts(self, wallet_account_id):
        response = self._execute(
            WalletAccountService.GET,
            f"{self.endpoint}/{wallet_account_id}/virtual_accounts",
            url=self.beta_url,
        )
        return [
            VirtualAccount(**virtual_account)
            for virtual_account in response.get("virtual_accounts", [])
        ]

    def withdraw_funds(self, wallet_account_id, **kwargs):
        data = WithdrawFundsRequest(**kwargs)
        response = self._execute(
            WalletAccountService.POST,
            f"{self.endpoint}/{wallet_account_id}/withdraw",
            data=data.dict(),
        )
        return Disbursement(**response["disbursements"])



import os
from datetime import datetime

from assembly_payments.services.bank_account import BankAccountService
from assembly_payments.services.company import CompanyService
from assembly_payments.services.direct_debit_authority import (
    DirectDebitAuthorityService,
)
from assembly_payments.services.disbursement import DisbursementService
from assembly_payments.services.item import ItemService
from assembly_payments.services.token import TokenService
from assembly_payments.services.user import UserService
from assembly_payments.services.virtual_account import VirtualAccountService
from assembly_payments.services.wallet_account import WalletAccountService
from assembly_payments.services.webhook import WebhookService

PROD_BASE_URL = "https://secure.api.promisepay.com"
PROD_AUTH_URL = "https://au-0000.auth.assemblypay.com"
PROD_BETA_URL = "https://au-0000.api.assemblypay.com"
TEST_BASE_URL = "https://test.api.promisepay.com"
TEST_AUTH_URL = "https://au-0000.sandbox.auth.assemblypay.com"
TEST_BETA_URL = "https://sandbox.au-0000.api.assemblypay.com"


class AssemblyClient:
    def __init__(
        self,
        client_id=None,
        client_secret=None,
        scope=None,
        grant_type=None,
        use_production=False,
        logging=False,
    ):
        self.client_id = client_id or os.environ.get("ASSEMBLY_CLIENT_ID", None)
        self.client_secret = client_secret or os.environ.get(
            "ASSEMBLY_CLIENT_SECRET", None
        )
        self.scope = scope or os.environ.get("ASSEMBLY_SCOPE", None)
        self.grant_type = grant_type or "client_credentials"
        self.logging = logging
        self.token = None

        self.base_url = TEST_BASE_URL
        self.auth_url = TEST_AUTH_URL
        self.beta_url = TEST_BETA_URL
        if use_production:
            self.base_url = PROD_BASE_URL
            self.auth_url = PROD_AUTH_URL
            self.beta_url = PROD_BETA_URL

        self._initialise_services()

    def get_auth(self):
        def refresh_token():
            return TokenService(
                base_url=self.base_url, auth_url=self.auth_url, logging=self.logging
            ).create(
                grant_type=self.grant_type,
                client_id=self.client_id,
                client_secret=self.client_secret,
                scope=self.scope,
            )

        if not self.token or self.token.expires_at <= datetime.now().timestamp():
            self.token = refresh_token()
        return self.token.access_token

    def _initialise_services(self):
        kwargs = dict(
            base_url=self.base_url,
            auth_url=self.auth_url,
            beta_url=self.beta_url,
            get_auth=self.get_auth,
            logging=self.logging,
        )
        self.users = UserService(**kwargs)
        self.companies = CompanyService(**kwargs)
        self.wallet_accounts = WalletAccountService(**kwargs)
        self.virtual_accounts = VirtualAccountService(**kwargs)
        self.bank_accounts = BankAccountService(**kwargs)
        self.disbursements = DisbursementService(**kwargs)
        self.items = ItemService(**kwargs)
        self.webhooks = WebhookService(**kwargs)
        self.direct_debit_authority = DirectDebitAuthorityService(**kwargs)



import jwt

from assembly_payments.services.base import BaseService
from assembly_payments.types import TokenRequest, Token


class TokenService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.endpoint = "/tokens"

    def create(self, **kwargs):
        data = TokenRequest(**kwargs)
        response = self._execute(TokenService.POST, self.endpoint, data=data.dict(), headers={}, url=self.auth_url)
        expires_at = jwt.decode(response.get('access_token'), algorithms=["RS256"], options={"verify_signature": False}).get("exp")
        return Token(**response, expires_at=expires_at)




from assembly_payments.services.base import BaseService
from assembly_payments.types import User, UserRequest, WalletAccount, BankAccount, SetDisbursementRequest


class UserService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.endpoint = "/users"

    def get(self, user_id):
        response = self._execute(UserService.GET, f"{self.endpoint}/{user_id}")
        return User(**response['users'])

    def list(self, *args, **kwargs):
        response = self._execute(UserService.GET, self.endpoint)
        return [User(**user) for user in response.get('users', [])]

    def create(self, **kwargs):
        data = UserRequest(**kwargs)
        response = self._execute(UserService.POST, self.endpoint, data=data.dict())
        return User(**response['users'])

    def get_wallet_account(self, user_id):
        response = self._execute(UserService.GET, f"{self.endpoint}/{user_id}/wallet_accounts")
        return WalletAccount(**response['wallet_accounts'])

    def get_bank_account(self, user_id):
        response = self._execute(UserService.GET, f"{self.endpoint}/{user_id}/bank_accounts")
        return BankAccount(**response['bank_accounts'])

    def set_disbursement_account(self, user_id, **kwargs):
        data = SetDisbursementRequest(**kwargs)
        response = self._execute(UserService.PATCH, f"{self.endpoint}/{user_id}/disbursement_account", data=data.dict())
        return User(**response['users'])




from assembly_payments.services.base import BaseService
from assembly_payments.types import Company, CompanyRequest


class CompanyService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.endpoint = "/companies"

    def list(self, *args, **kwargs):
        response = self._execute(CompanyService.GET, self.endpoint)
        return [Company(**company) for company in response.get('companies', [])]

    def create(self, **kwargs):
        data = CompanyRequest(**kwargs)
        response = self._execute(CompanyService.POST, self.endpoint, data=data.dict())
        return Company(**response['companies'])



from assembly_payments.services.base import BaseService
from assembly_payments.types import DirectDebitAuthority, DirectDebitAuthorityRequest


class DirectDebitAuthorityService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.endpoint = "/direct_debit_authorities"

    def get(self, debit_authority_id):
        response = self._execute(
            DirectDebitAuthorityService.GET, f"{self.endpoint}/{debit_authority_id}"
        )
        return DirectDebitAuthority(**response["direct_debit_authorities"])

    def list(self, *args, **kwargs):
        response = self._execute(DirectDebitAuthorityService.GET, self.endpoint)
        return [
            DirectDebitAuthority(**debit_authority)
            for debit_authority in response.get("direct_debit_authorities", [])
        ]

    def create(self, **kwargs):
        data = DirectDebitAuthorityRequest(**kwargs)
        response = self._execute(
            DirectDebitAuthorityService.POST, self.endpoint, data=data.dict()
        )
        return DirectDebitAuthority(**response["direct_debit_authorities"])




from assembly_payments.services.base import BaseService
from assembly_payments.types import Webhook, WebhookRequest, WebhookUpdateRequest


class WebhookService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.endpoint = "/webhooks"

    def get(self, webhook_id):
        response = self._execute(WebhookService.GET, f"{self.endpoint}/{webhook_id}", url=self.beta_url)
        return Webhook(**response['webhooks'])

    def list(self, *args, **kwargs):
        response = self._execute(WebhookService.GET, self.endpoint, url=self.beta_url)
        return [Webhook(**webhook) for webhook in response.get('webhooks', [])]

    def create(self, **kwargs):
        data = WebhookRequest(**kwargs)
        response = self._execute(WebhookService.POST, self.endpoint, data=data.dict(exclude_none=True), url=self.beta_url)
        return Webhook(**response)

    def update(self, webhook_id, **kwargs):
        data = WebhookUpdateRequest(**kwargs)
        response = self._execute(WebhookService.PATCH, f"{self.endpoint}/{webhook_id}", data=data.dict(exclude_none=True), url=self.beta_url)
        return Webhook(**response['webhooks'])

    def delete(self, webhook_id):
        self._execute(WebhookService.DELETE, f"{self.endpoint}/{webhook_id}", url=self.beta_url)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This setup.py is based off https://github.com/kennethreitz/setup.py
import io
import os
import pathlib
import sys
from shutil import rmtree

from setuptools import setup, Command, find_packages

# Package meta-data.
NAME = 'py-assembly-payments'
DESCRIPTION = 'Python wrapper around the Assembly Payments API'
URL = 'https://github.com/divipayhq/py-assembly-payments'
AUTHOR = 'Matthew Egan <DiviPay>'
VERSION = '0.4.0'

REQUIRED = []

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

about = {'__version__': VERSION}

class UploadCommand(Command):
    """Support setup.py publish."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(HERE, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(about['__version__']))
        os.system('git push --tags')

        sys.exit()

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=README,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    url=URL,
    license="MIT",
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    install_requires=[
        "pydantic", "PyJWT", "requests"
    ],
    cmdclass={
        'upload': UploadCommand,
    },
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
    ],
)



from assembly_payments.services.base import BaseService
from assembly_payments.types import VirtualAccount, VirtualAccountRequest


class VirtualAccountService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.endpoint = "/virtual_accounts"

    def get(self, virtual_account_id):
        response = self._execute(VirtualAccountService.GET, f"{self.endpoint}/{virtual_account_id}", url=self.beta_url)
        return VirtualAccount(**response)

    def create(self, *args, **kwargs):
        data = VirtualAccountRequest(**kwargs).dict()
        wallet_account_id = data.pop("wallet_account_id")
        response = self._execute(VirtualAccountService.POST, f"/wallet_accounts/{wallet_account_id}/virtual_accounts", data=data, url=self.beta_url)
        return VirtualAccount(**response)



from json import JSONDecodeError

import requests

from assembly_payments.exceptions import PyAssemblyPaymentsNotImplementedException, PyAssemblyPaymentsBadRequest, \
    PyAssemblyPaymentsUnprocessableEntity, PyAssemblyPaymentsForbidden, PyAssemblyPaymentsNotFound, \
    PyAssemblyPaymentsConflict


class BaseService:
    GET = 'get'
    POST = 'post'
    PATCH = 'patch'
    DELETE = 'delete'

    def __init__(self, get_auth=None, base_url=None, auth_url=None, beta_url=None, logging=False):
        self.get_auth = get_auth
        self.endpoint = None
        self.base_url = base_url
        self.auth_url = auth_url
        self.beta_url = beta_url
        self.logging = logging

    def _execute(self, method, endpoint, data=None, headers=None, url=None):
        if headers is None:
            headers = dict(
                Authorization=f"Bearer {self.get_auth()}"
            )

        if url is None:
            url = self.base_url

        response = getattr(requests, method)(f"{url}{endpoint}", json=data, headers=headers)

        if self.logging:
            print(method.upper(), endpoint, response.status_code, response.text)

        self._handle_exceptions(response)
        return response.json()

    def _handle_exceptions(self, response):
        exc_classes = {
            400: PyAssemblyPaymentsBadRequest,
            403: PyAssemblyPaymentsForbidden,
            404: PyAssemblyPaymentsNotFound,
            409: PyAssemblyPaymentsConflict,
            422: PyAssemblyPaymentsUnprocessableEntity,
        }

        exc_class = exc_classes.get(response.status_code)
        if exc_class:
            try:
                data = response.json()
            except JSONDecodeError:
                data = response.text
            raise exc_class(data)

    def list(self, *args, **kwargs):
        raise PyAssemblyPaymentsNotImplementedException(f"{self.__class__} does not implement list. Please raise an issue or PR if you'd like it implemented.")

    def get(self, *args, **kwargs):
        raise PyAssemblyPaymentsNotImplementedException(f"{self.__class__} does not implement get. Please raise an issue or PR if you'd like it implemented.")

    def create(self, **kwargs):
        raise PyAssemblyPaymentsNotImplementedException(f"{self.__class__} does not implement create. Please raise an issue or PR if you'd like it implemented.")

    def update(self, **kwargs):
        raise PyAssemblyPaymentsNotImplementedException(f"{self.__class__} does not implement update. Please raise an issue or PR if you'd like it implemented.")

    def delete(self, *args, **kwargs):
        raise PyAssemblyPaymentsNotImplementedException(f"{self.__class__} does not implement delete. Please raise an issue or PR if you'd like it implemented.")
